using System.Collections;
using System.Collections.Generic;
using PixelLiner.PixelLinerLib;
#if UNITY_EDITOR
using UnityEditor;
#endif
using UnityEngine;
namespace PixelLiner {
    [RequireComponent (typeof (MeshFilter))]
    [RequireComponent (typeof (MeshRenderer))]
    public class PxlCharaAnimator : BasicAnimator {

        public string chara_title;
        public string pose_title;
        private float offsetPixelX_ = 0;
        private float offsetPixelY_ = 0;
        public AIM pose_aim;
        [System.NonSerialized]
        public MeshFilter TargetMeshFilter;
        [System.NonSerialized]
        public MeshRenderer TargetMeshRenderer;
        // ============================

        public static string shader_default = "PixelLiner/Normal";

        public const AIM AIM_L = AIM.L; // 0
        public const AIM AIM_R = AIM.R; // 1
        public const AIM AIM_T = AIM.T; // 2
        public const AIM AIM_B = AIM.B; // 3
        public const AIM AIM_LT = AIM.LT; // 4
        public const AIM AIM_TR = AIM.TR; // 5
        public const AIM AIM_BL = AIM.BL; // 6
        public const AIM AIM_RB = AIM.RB; // 7
        // ============================

        protected PxlCharacter pChar;
        protected PxlPose pPose;
        protected PxlSequence pSq;
        protected Texture2D pTexture;
        Material[] AMtr = new Material[1];
        protected Material Mtr;
        Color32 ColorForMaterial = new Color32 (255, 255, 255, 255);
        bool is_chara_loading = false;
        protected int pre_loop_count;
        [SerializeField, HideInInspector]
        int pose_jump_flag = 1; // AI.allow_pose_jump　= true/false で設定

        public void setChara (string title) {
            chara_title = title;
            pTexture = null;
            PxlCharacter Pre = pChar;
            pChar = PxlsLoader.getPxlCharacter (chara_title);
            if (pChar != null && Pre != pChar) {
                if (!pChar.isLoadCompleted ()) {
                    is_chara_loading = true;
                    pPose = null;
                    pSq = null;
                    return;
                }
                setPose (pose_title, 1);
            }
        }
        public virtual void setPose (string title, int restart_anim = -1) {
            // restart_anim: -1 ならば、アニメに変更があるときのみアニメーションをはじめから再生します
            // restart_anim: -1000 を与えるとアニメの変更を省略します
            if ((title == pose_title && restart_anim <= 0) || title == "" || title == null) return;
            pose_title = title;
            PxlPose Pre = pPose;
            if (pChar != null && !is_chara_loading) {
                var PoseElement = pChar.getPoseByName (title);
                if (PoseElement == null) {
                    Debug.Log ("PxlCharaAnimator: ポーズが見つかりません" + title);
                    PoseElement = pChar.getPose (0);
                }
                setPoseAfter (PoseElement, Pre, restart_anim);
            }
        }
        public void setPose (PxlPose PoseElement, int restart_anim = -1) {
            PxlPose Pre = pPose;
            if ((PoseElement == Pre && restart_anim <= 0) || PoseElement == null) return;
            pose_title = PoseElement.title;
            if (pChar != PoseElement.pChar) {
                pChar = PoseElement.pChar;
                chara_title = pChar.title;
            }
            if (pChar != null && !is_chara_loading) setPoseAfter (PoseElement, Pre, restart_anim);

        }
        private void setPoseAfter (PxlPose PoseElement, PxlPose Pre, int restart_anim = -1) {
            pPose = PoseElement;
            Texture2D _pTexture = pPose.getSequence (0).getFrame (0).getImageTexture ();
            if (pTexture != _pTexture) {
                pTexture = _pTexture;
                AMtr[0] = prepareMaterial (ref Mtr, pTexture);
                Mtr.mainTexture = pTexture;
                TargetMeshRenderer.sharedMaterials = AMtr;
            }
            if (restart_anim == -1000) return;
            if (restart_anim <= -1) restart_anim = Pre != pPose ? 1 : 0;
            if (pPose != null && (pPose != Pre || restart_anim == 1)) {
                setAim (pose_aim, restart_anim);
            }
        }
        public void setAim (AIM aim, int restart_anim = -1) {
            // restart_anim: -1 ならば、アニメに変更があるときのみアニメーションをはじめから再生します
            setAim ((int) aim, restart_anim);
        }
        public void setAim (int aim, int restart_anim = -1) {
            // restart_anim: -1 ならば、アニメに変更があるときのみアニメーションをはじめから再生します
            if ((aim == (int) pose_aim && restart_anim <= 0)) return;
            pose_aim = (AIM) aim;
            PxlSequence Pre = pSq;
            if (pPose != null && !is_chara_loading) {
                pose_jump_flag = (pose_jump_flag >= 1) ? 1 : 0;
                pSq = pPose.getSequence ((int) aim);
                if (restart_anim == -1) restart_anim = Pre != pSq ? 1 : 0;
                if (pSq != null)
                    if (pSq != Pre) {
                        loop_to = pSq.loop_to;
                        maisu = pSq.countFrames ();
                        calculateFrameSpeed ();
                        animReset (restart_anim == 1 ? 0 : X.MMX (0, cframe_, maisu - 1));
                    }
                else if (restart_anim == 1) {
                    // animReset
                    animReset (0);
                }
                pre_loop_count = -1;
            }
        }
        public virtual void Start () {
            if (TargetMeshFilter == null) {
                TargetMeshFilter = GetComponent<MeshFilter> ();
            }
            if (TargetMeshRenderer == null) {
                TargetMeshRenderer = GetComponent<MeshRenderer> ();
            }
            if (chara_title != "" && pChar == null) {
                setChara (chara_title);
            }
        }
        protected override void Update () {
            if (is_chara_loading) {
                if (pChar == null) return;
                if (!pChar.isLoadCompleted ()) return;
                is_chara_loading = false;
                setPose (pose_title, 1);
            }
            if (pSq == null) return;
            base.Update ();
            if (TargetMeshFilter == null) return;
            if (isChanged () || loop_count != pre_loop_count) {
                if (pose_jump_flag >= 2 || (pose_jump_flag >= 0 && get_loop_count () >= pPose.end_jump_loop_count)) {
                    // ちょうどループした
                    if (pose_jump_flag <= 0) pose_jump_flag = -2;
                    else {
                        if (pPose.end_jump_title != "" && pPose.end_jump_title != null) {
                            setPose (pPose.end_jump_title, 1);
                            updateAnimator (0);
                        }
                    }
                }
                fineCurrentFrameMesh ();
                pre_loop_count = loop_count;
            }
        }
        public virtual bool fineCurrentFrameMesh (bool force_rewrite = false) {
            if (pSq == null) return false;
            Mesh dMesh = pSq.getFrame (cframe_).getDrawnMesh (false, offsetPixelX_, offsetPixelY_);
            if (force_rewrite || TargetMeshFilter.sharedMesh != dMesh) {
                TargetMeshFilter.sharedMesh = dMesh;
                return true;
            }
            return false;
        }
        public virtual Material prepareMaterial (ref Material Mtr, Texture TargetTexture) {
            if (Mtr == null) Mtr = new Material (Shader.Find (shader_default));
            else Mtr.mainTexture = TargetTexture;
            return Mtr;
        }
        public Color32 color {
            get { return ColorForMaterial; }
            set {
                ColorForMaterial = value;;
                if (TargetMeshRenderer != null && Mtr != null) {
                    Mtr.SetColor ("_Color", value);
                    TargetMeshRenderer.sharedMaterials = AMtr;
                }
            }
        }
        public float alpha {
            get { return ColorForMaterial.a / 255f; }
            set {
                ColorForMaterial.a = (byte) (X.MMX (0, (int) (value * 255f), 255));
                if (TargetMeshRenderer != null && Mtr != null) {
                    Mtr.SetColor ("_Color", ColorForMaterial);
                    TargetMeshRenderer.sharedMaterials = AMtr;
                }
            }
        }
        public PxlCharacter getCurrentCharacter () {
            return pChar;
        }
        public PxlFrame getCurrentDrawnFrame () {
            return pSq != null?pSq.getFrame (cframe_) : null;
        }

        public PxlPose getCurrentPose () {
            return pPose;
        }
        public PxlSequence getCurrentSequence () {
            return pSq;
        }
        public Texture2D getCurrentTexture () {
            return pTexture;
        }
        public Material getRendererMaterial () {
            return Mtr;
        }
        public bool isCharacterLoadFinished () { return !is_chara_loading; }
        public void setRendererMaterial (Material ReplaceMaterial) {
            Mtr = ReplaceMaterial;
            AMtr[0] = ReplaceMaterial;

            TargetMeshRenderer.sharedMaterials = AMtr;
        }

        public float offsetPixelX {
            get { return offsetPixelX_; }
            set {
                if (offsetPixelX_ != value) {
                    offsetPixelX_ = value;
                    fineCurrentFrameMesh (true);
                }
            }
        }
        public float offsetPixelY {
            get { return offsetPixelY_; }
            set {
                if (offsetPixelY_ != value) {
                    offsetPixelY_ = value;
                    fineCurrentFrameMesh (true);
                }
            }
        }
        public void setOffsetPixel (float pixel_x, float pixel_y) {
            offsetPixelX_ = pixel_x;
            offsetPixelY_ = pixel_y;
            fineCurrentFrameMesh (true);
        }
        public bool poseIs (params string[] Astr) {
            if (pPose == null) return false;
            int ARMX = Astr.Length;
            for (int i = 0; i < ARMX; i++) {
                if (pPose.title == Astr[i]) return true;
            }
            for (int i = 0; i < ARMX; i++) {
                if (pChar.getPoseByName (Astr[i]) == pPose) return true;
            }
            return false;
        }
        // ======================================================== 
        // MARK: inner 
        // ========================================================
        protected override int getAf (int i) { // フレーム単独の表示時間取得
            if (pSq == null) return 0;
            return pSq.getFrame (i).crf60;
        }
        protected override int getEndAf (int i) { // フレーム単独の表示時間取得
            if (i < 0 || pSq == null) return 0;
            return pSq.getFrame (i).endf60;
        }
        protected override void setAf (int i, int _f) { // そのフレームの累計表示時間取得
            if (pSq == null) return;
            pSq.getFrame (i).crf60 = _f;
        }
        protected override void setEndAf (int i, int _f) { // そのフレームの累計表示時間取得
            if (pSq == null) return;
            pSq.getFrame (i).endf60 = _f;
        }

        public bool allow_pose_jump {
            get { return pose_jump_flag > 0; }
            set {
                if (value) {
                    pose_jump_flag = (pose_jump_flag < 0) ? 2 : 1;
                } else pose_jump_flag = (pose_jump_flag >= 2) ? -2 : 0;

            }
        }
        public bool looped_in_preveious_frame {
            get { return cframe_is_looped; }
        }
        public bool looped_already {
            get { return loop_count > 0; }
        }
    }

#if UNITY_EDITOR
    [CustomEditor (typeof (PxlCharaAnimator))]
    public class PxlCharaAnimatorEditor : Editor {
        public override void OnInspectorGUI () {
            var EF = target as PxlCharaAnimator;

            EF.chara_title =
                EditorGUILayout.TextField ("CharacterTitle", EF.chara_title);
            EF.pose_title =
                EditorGUILayout.TextField ("PoseTitle", EF.pose_title);
            EF.pose_aim = (AIM) (
                EditorGUILayout.Popup ("Default Aim", (int) EF.pose_aim, CAim.Astring));
            EF.allow_pose_jump =
                EditorGUILayout.Toggle ("Allow Pose Jump", EF.allow_pose_jump);
            Color32 _color = (
                EditorGUILayout.ColorField ("Color", EF.color));
            if (!_color.Equals (EF.color)) EF.color = _color;

            MeshRenderer Mrd = EF.GetComponent<MeshRenderer> ();
            if (Mrd != null) {

                Material Mtr = Mrd.sharedMaterial;
                if (Mtr == null) {
                    EditorGUILayout.HelpBox ("Material in MeshRenderer does not exists!", MessageType.Warning);
                } else {
                    if (!Mtr.HasProperty ("_MainTex")) {
                        EditorGUILayout.HelpBox ("Material in MeshRenderer does not have a _MainTex texture property.", MessageType.Warning);
                    }
                }
            }
        }
    }
#endif

}