using System.Collections;
using System.Collections.Generic;
using PixelLiner.PixelLinerLib;
using UnityEngine;

// canvas や camera など、常に存在するオブジェクトにつけて管理するとよいでしょう。
// ロードが終わったら Component および Component がくっついている GameObject 自体は破壊しても良い。

namespace PixelLiner {

    public class PxlsLoader : MonoBehaviour {
        [System.Serializable]
        struct PxlsLoaderItems {
            public TextAsset assetFile;
            public string title;
            [SerializeField]
            public Texture2D[] ExternalPng;
        }

        static PxlsLoader Instance;
        public static bool deny_overload = true; // 同じタイトルのファイルを読み込まない
        [SerializeField]
        PxlsLoaderItems[] Targets;
        public float pixelsPerUnit = 16.0f;
        public bool autoFlipX = true;
        public static float loadSpeed = 1;
        public static bool texture_unreadable = true; // pxls から作成された Texture2D を読み込めないようにしメモリの使用を抑えます。
        // ========================================================
        // ========================================================
        // ========================================================
        // ======================================================== 
        // MARK: admin 
        // ========================================================
        static Dictionary<string, PxlCharacter> OPxlChara; // 保持用
        static List<PxlCharacter> ALoading;
        public static void assignChara (PxlCharacter C) {
            if (OPxlChara == null) OPxlChara = new Dictionary<string, PxlCharacter> ();
            if (!OPxlChara.ContainsKey (C.title)) {
                OPxlChara[C.title] = C;
            }
        }
        public static PxlCharacter getPxlCharacter (string title) {
            if (OPxlChara == null) return null;
            if (OPxlChara.ContainsKey (title)) {
                return OPxlChara[title];
            }
            return null;
        }
        public static void disposeCharacter (string title, bool dispose_image = true) {
            var Pxl = getPxlCharacter (title);
            if (Pxl != null) {
                OPxlChara.Remove (title);
                Pxl.Destroy (!dispose_image);
            }
        }
        public static PxlCharacter loadCharacterASyncFromResources (string title, string TextAssetPath, bool loadExternalTexture, float _pixelsPerUnit, bool autoFlipX = true) {
            PxlCharacter C = PxlsLoader.loadCharacterASync (title, Resources.Load<TextAsset> (TextAssetPath), _pixelsPerUnit, autoFlipX);
            if (C != null) {
                C.external_png_header = PxlsTexture.PxlsTexturePath (TextAssetPath, -1);
                if (loadExternalTexture) {
                    int i = 0;
                    List<PxlsTexture> ATx = null;
                    while (true) {
                        PxlsTexture Ptx = new PxlsTexture (PxlsTexture.PxlsTexturePath (TextAssetPath, i++));
                        if (Ptx.Image == null) break;
                        if (ATx == null) {
                            ATx = new List<PxlsTexture> (4);
                        }
                        ATx.Add (Ptx);
                    }
                    if (ATx != null) {
                        C.AddExternalPng (ATx.ToArray ());
                    }
                }
            }
            return C;
        }
        public static PxlCharacter loadCharacterASync (string title, TextAsset Ta, float _pixelsPerUnit, bool autoFlipX = true) {
            if (Ta == null) return null;
            return loadCharacterASync (title, Ta.bytes, null, _pixelsPerUnit, autoFlipX);
        }
        public static PxlCharacter loadCharacterASync (string title, TextAsset Ta, Texture2D[] ExternalTextureArray, float _pixelsPerUnit, bool autoFlipX = true) {
            if (Ta == null) return null;
            return loadCharacterASync (title, Ta.bytes, PxlsTexture.ConvertTextureArray (ExternalTextureArray), _pixelsPerUnit, autoFlipX);
        }
        public static PxlCharacter loadCharacterASync (string title, byte[] Ta, float _pixelsPerUnit, bool autoFlipX = true) {
            if (Ta == null) return null;
            return loadCharacterASync (title, Ta, null, _pixelsPerUnit, autoFlipX);
        }
        public static PxlCharacter loadCharacterASync (string title, byte[] Ta, PxlsTexture[] ExternalTextureArray, float _pixelsPerUnit, bool autoFlipX = true) {
            if (OPxlChara == null) OPxlChara = new Dictionary<string, PxlCharacter> ();
            if (title == "") {
                title = "Character" + (OPxlChara.Count);
            }
            if (OPxlChara.ContainsKey (title)) {
                Debug.Log ("Character " + title + " already exists!");
                return null;
            }
            PxlCharacter C = new PxlCharacter (title);
            C.pixelsPerUnit = _pixelsPerUnit;
            C.autoFlipX = autoFlipX;
            if (C.loadASync (Ta)) {
                C.AddExternalPng (ExternalTextureArray);
                return C;
            }
            return null;
        }
        public static PxlCharacter reloadCharacterASync (string title, byte[] Ta) {
            if (!OPxlChara.ContainsKey (title)) {
                Debug.Log ("Character " + title + " does not exists!");
                return null;
            }
            PxlCharacter pC = OPxlChara[title];
            pC.Destroy ();
            OPxlChara.Remove (title);
            PxlCharacter C = loadCharacterASync (title, Ta, pC.getExternalTextureArray (), pC.pixelsPerUnit, pC.autoFlipX);
            if (C == null) {
                Debug.Log ("Unknown error: loading pxls \"" + title + "\"");
                OPxlChara[title] = pC;
                return pC;
            }
            return C;

        }
        public static bool isLoadCompletedAll () {
            if (Instance == null || Instance.Targets != null) return true;
            if (OPxlChara == null) return true;
            foreach (KeyValuePair<string, PxlCharacter> _kvp in OPxlChara) {
                //string _k = kvp.Key;
                //THETYPE _val = kvp.Value;
                if (!_kvp.Value.isLoadCompleted ()) return false;
            }
            return true;
        }
        // for instance
        public void Awake () {
            Instance = this;
            if (Targets != null) {
                if (Targets.Length > 0) {
                    int ARMX = Targets.Length;
                    for (int i = 0; i < ARMX; i++) {
                        PxlsLoaderItems Tx = Targets[i];
                        if (deny_overload && getPxlCharacter (Tx.title) != null) {
                            Debug.Log ("Character \"" + Tx.title + "\" already loaded");
                            continue;
                        }
                        if (Tx.assetFile != null) {
                            PxlCharacter C = loadCharacterASync (Tx.title, Tx.assetFile, Tx.ExternalPng, this.pixelsPerUnit, this.autoFlipX);
                        } else {
                            Debug.Log ("Asset file \"" + Tx.title + "\" does not exists on resources");
                        }
                    }
                }
                Targets = null;
            }
            enabled = false;
        }
        public static void initLoadASync (PxlCharacter C) {
            if (ALoading == null) {
                ALoading = new List<PxlCharacter> (1);
                ALoading.Add (C);
                if (Instance == null) {
                    Camera Attach_to = Camera.main;
                    Instance = (Attach_to == null? new GameObject ("__PxlsLoader_Instance") : Attach_to.gameObject).AddComponent<PxlsLoader> ();
                }
                Instance.enabled = true;
                Instance.StartCoroutine ("loadProgressCoroutine");
            } else if (ALoading.IndexOf (C) == -1) {
                ALoading.Add (C);
            }
        }
        IEnumerator loadProgressCoroutine () {
            yield return null;
            int p = 1;
            float _loadSpeed = loadSpeed / ALoading.Count;
            while (true) {
                p = 0;
                int i = 0;
                while (i < ALoading.Count) {
                    PxlCharacter C = ALoading[i];
                    if (C != null) {
                        if (C.loadProgress (_loadSpeed)) ALoading[i] = null;
                        else p++;
                    }
                    i++;
                }
                yield return null;
                if (p > 0) {
                    _loadSpeed = loadSpeed / p;
                } else if (i == ALoading.Count) break;
            }
            ALoading = null;
            Instance.enabled = false;
        }

    }
}