using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using UnityEngine;
/* basic lib */
namespace PixelLiner.PixelLinerLib {
    public class ByteArray {
        public byte[] bytes;
        private ulong bytes_length;
        static byte[] Arev;
        public ulong position;
        public const string ERROR_EOF = "ERROR_EOF";
        public ByteArray (uint alloc = 0) {
            if (Arev == null) Arev = new byte[8];
            bytes = new byte[alloc];
        }
        public ulong Length {
            get { return (ulong) bytes_length; }
        }
        public ulong bytesAvailable {
            get { return (ulong) (bytes_length - position); }
        }
        public ByteArray (ByteArray _B, bool position_set_to_foot = false) {
            if (Arev == null) Arev = new byte[8];
            bytes = new byte[_B.Length];
            writeBytes (_B);
            if (!position_set_to_foot) position = 0;
        }

        public ByteArray (byte[] _bytes, bool position_set_to_foot = false) {
            if (Arev == null) Arev = new byte[8];
            bytes = new byte[_bytes.Length];
            bytes_length = (ulong) bytes.Length;
            Array.Copy (_bytes, 0, bytes, 0, _bytes.Length);
            if (Arev == null) Arev = new byte[8];
            if (position_set_to_foot) position = bytes_length;
        }
        private int rbyte () {
            if (position >= (ulong) bytes.Length) {
                throw (new System.Exception (ERROR_EOF));
            }
            int r = bytes[position++];
            return r;
        }
        // read
        public bool readBoolean () {
            bool r = rbyte () != 0;
            return r;
        }
        public char readChar () {
            char r = (char) rbyte ();
            return r;
        }
        public int readByte () {
            return (int) readChar ();
        }
        public byte readUByte () {
            return bytes[position++];
        }
        public short readShort () {
            // big endian
            Arev[1] = (byte) rbyte ();
            Arev[0] = (byte) rbyte ();
            return BitConverter.ToInt16 (Arev, 0);
        }
        public ushort readUShort () {
            Arev[1] = (byte) rbyte ();
            Arev[0] = (byte) rbyte ();
            return BitConverter.ToUInt16 (Arev, 0);
        }
        public int readInt () {
            Arev[3] = (byte) rbyte ();
            Arev[2] = (byte) rbyte ();
            Arev[1] = (byte) rbyte ();
            Arev[0] = (byte) rbyte ();
            return BitConverter.ToInt32 (Arev, 0);
        }
        public uint readUInt () {
            Arev[3] = (byte) rbyte ();
            Arev[2] = (byte) rbyte ();
            Arev[1] = (byte) rbyte ();
            Arev[0] = (byte) rbyte ();
            return BitConverter.ToUInt32 (Arev, 0);
        }
        public uint readUnsignedInt () {
            return readUInt ();
        }
        public ulong readULong () {
            Arev[7] = (byte) rbyte ();
            Arev[6] = (byte) rbyte ();
            Arev[5] = (byte) rbyte ();
            Arev[4] = (byte) rbyte ();
            Arev[3] = (byte) rbyte ();
            Arev[2] = (byte) rbyte ();
            Arev[1] = (byte) rbyte ();
            Arev[0] = (byte) rbyte ();
            return BitConverter.ToUInt64 (Arev, 0);
        }
        public float readFloat () {
            Arev[3] = (byte) rbyte ();
            Arev[2] = (byte) rbyte ();
            Arev[1] = (byte) rbyte ();
            Arev[0] = (byte) rbyte ();
            float r = BitConverter.ToSingle (Arev, 0);
            return r;
        }
        public double readDouble () {
            Arev[7] = (byte) rbyte ();
            Arev[6] = (byte) rbyte ();
            Arev[5] = (byte) rbyte ();
            Arev[4] = (byte) rbyte ();
            Arev[3] = (byte) rbyte ();
            Arev[2] = (byte) rbyte ();
            Arev[1] = (byte) rbyte ();
            Arev[0] = (byte) rbyte ();
            double r = BitConverter.ToDouble (Arev, 0);
            return r;
        }
        public DateTime readDate () {
            DateTime D = (new System.DateTime (2020, 8, 28, 8, 28, 0, 0));

            return D.AddSeconds (readDouble ());
        }
        public void readByteA (ref byte[] Af, bool unsigned_flag = false) {
            int ARMX = readUShort ();
            if (Af.Length < ARMX) {
                System.Array.Resize (ref Af, ARMX);
            }
            for (int i = 0; i < ARMX; i++) {
                if (unsigned_flag) Af[i] = readUByte ();
                else Af[i] = (byte) readByte ();
            }
        }
        public void readFloatA (ref float[] Af) {
            int ARMX = readUShort ();
            if (Af.Length < ARMX) {
                System.Array.Resize (ref Af, ARMX);
            }
            for (int i = 0; i < ARMX; i++) {
                Af[i] = readFloat ();
            }
        }
        public void readIntA (ref int[] Af) {
            int ARMX = readUShort ();
            if (Af.Length < ARMX) {
                System.Array.Resize (ref Af, ARMX);
            }
            for (int i = 0; i < ARMX; i++) {
                Af[i] = readInt ();
            }
        }
        public string readMultiByte (int len, string encoding = "utf-8") {
            string r;
            if (encoding == "utf-8") {
                r = System.Text.Encoding.UTF8.GetString (bytes, (int) position, len);
                position += (ulong) len;
                return r;
            }
            r = System.Text.Encoding.ASCII.GetString (bytes, (int) position, len);
            position += (ulong) len;
            return r;
        }
        public string readPascalString (string encoding = "utf-8") {
            byte len = readUByte ();
            if (len == 0) return "";
            string str = readMultiByte (len, encoding);
            return str;
        }
        public string readString (string encoding = "utf-8") {
            ushort len = readUShort ();
            if (len == 0) return "";
            string str = readMultiByte (len, encoding);
            return str;
        }
        public void readBytes (ByteArray DestBytes, uint dest_offset, uint length) {
            DestBytes.alloc (dest_offset + length, true);
            if (length == 0) return;
            do {
                DestBytes.bytes[dest_offset++] = (byte) rbyte ();
            }
            while (--length > 0);
        }
        public Dictionary<string, int> readDictionaryStrInt () {
            Dictionary<string, int> O = new Dictionary<string, int> ();
            int ARMX = readUShort ();
            string k;
            for (int i = 0; i < ARMX; i++) {
                k = readString ();
                O[k] = readInt ();
            }
            return O;
        }
        public Dictionary<string, byte> readDictionaryStrByte () {
            Dictionary<string, byte> O = new Dictionary<string, byte> ();
            int ARMX = readUShort ();
            string k;
            for (int i = 0; i < ARMX; i++) {
                k = readString ();
                O[k] = (byte) rbyte ();
            }
            return O;
        }
        public Dictionary<T, float> readDictionaryTFloat<T> (Func<int, T> FnConvertInt2T) {
            var O = new Dictionary<T, float> ();
            int ARMX = readUShort ();
            for (int i = 0; i < ARMX; i++) {
                T key = FnConvertInt2T (readInt ());
                O[key] = readFloat ();
            }
            return O;
        }
        public Dictionary<T, int> readDictionaryTInt<T> (Func<int, T> FnConvertInt2T) {
            var O = new Dictionary<T, int> ();
            int ARMX = readUShort ();
            for (int i = 0; i < ARMX; i++) {
                T key = FnConvertInt2T (readInt ());
                O[key] = readInt ();
            }
            return O;
        }
        public ByteArray readExtractBytes (int len_count = 4) {
            uint len;
            if (len_count == 4) len = readUInt ();
            else if (len_count == 2) len = readUShort ();
            else len = readUByte ();
            ByteArray Ba = new ByteArray (len);
            readBytes (Ba, 0, len);
            return Ba;
        }
        public ByteArray readExtractBytesShifted (int len_count = 4) {
            uint len;
            if (len_count == 4) len = readUInt ();
            else if (len_count == 2) len = readUShort ();
            else len = readUByte ();
            byte sh = (byte) readByte ();
            ByteArray Ba = new ByteArray (len);
            for (int i = 0; i < len; i++) {
                Ba.bytes[i] = (byte) ((rbyte () + 0x100 - sh) & 0xff);
            }
            Ba.position = 0;

            return Ba;
        }
        public void alloc (ulong length, bool extend_length = false) {
            if ((ulong) bytes.Length < length) {
                System.Array.Resize (ref bytes, (int) length);
                if (extend_length) {
                    bytes_length = X.Mx (bytes_length, (ulong) length);
                }
            }
        }

        // ======================================================== 
        // MARK: writing 
        // ========================================================
        public void writeByte (int b) {
            if (position >= (ulong) bytes.Length) {
                alloc ((ulong) bytes.Length + 512);
            }
            bytes[position++] = (byte) (b);
            bytes_length = X.Mx (bytes_length, (ulong) position);
        }
        public void writeBool (Boolean b) {
            writeByte (b?1 : 0);
        }
        public void writeUInt (uint b) {
            writeByte ((int) ((b >> 24) & 255));
            writeByte ((int) ((b >> 16) & 255));
            writeByte ((int) ((b >> 8) & 255));
            writeByte ((int) ((b) & 255));
        }
        public void writeULong (ulong b) {
            writeByte ((int) ((b >> 56) & 255));
            writeByte ((int) ((b >> 48) & 255));
            writeByte ((int) ((b >> 40) & 255));
            writeByte ((int) ((b >> 32) & 255));
            writeByte ((int) ((b >> 24) & 255));
            writeByte ((int) ((b >> 16) & 255));
            writeByte ((int) ((b >> 8) & 255));
            writeByte ((int) ((b) & 255));
        }
        public void writeInt (int b) {
            writeUInt ((uint) b);
        }
        public void writeShort (short b) {
            writeByte ((int) ((b >> 8) & 255));
            writeByte ((int) ((b) & 255));
        }
        public void writeUShort (ushort b) {
            writeByte ((int) ((b >> 8) & 255));
            writeByte ((int) ((b) & 255));
        }
        public void writeFloat (float f) {
            byte[] Aba = BitConverter.GetBytes (f);
            writeBytesReverse (Aba);
        }
        public void writeDouble (double f) {
            byte[] Aba = BitConverter.GetBytes (f);
            writeBytesReverse (Aba);
        }
        public void writeDate (DateTime D) {
            var Dif = D - (new System.DateTime (2020, 8, 28, 8, 28, 0, 0));
            writeDouble (Dif.TotalSeconds);
        }
        public void writeByteA (byte[] Af) {
            int ARMX = Af.Length;
            writeUShort ((ushort) ARMX);
            for (int i = 0; i < ARMX; i++) {
                writeByte (Af[i]);
            }
        }
        public void writeFloatA (float[] Af) {
            int ARMX = Af.Length;
            writeUShort ((ushort) ARMX);
            for (int i = 0; i < ARMX; i++) {
                writeFloat (Af[i]);
            }
        }
        public void writeIntA (int[] Af) {
            int ARMX = Af.Length;
            writeUShort ((ushort) ARMX);
            for (int i = 0; i < ARMX; i++) {
                writeInt (Af[i]);
            }
        }
        public void writeString (string text, string encoding = "utf-8") {
            if (text == "" || text == null) {
                writeUShort (0);
                return;
            }
            byte[] data;
            if (encoding == "utf-8") {
                data = System.Text.Encoding.UTF8.GetBytes (text);
            } else {
                data = System.Text.Encoding.ASCII.GetBytes (text);
            }
            if (data.Length > ushort.MaxValue) System.Array.Resize (ref data, ushort.MaxValue);
            writeUShort ((ushort) data.Length);
            writeBytes (data);
        }
        public void writePascalString (string text, string encoding = "utf-8") {
            if (text == "" || text == null) {
                writeByte (0);
                return;
            }
            byte[] data;
            if (encoding == "utf-8") {
                data = System.Text.Encoding.UTF8.GetBytes (text);
            } else {
                data = System.Text.Encoding.ASCII.GetBytes (text);
            }
            if (data.Length > byte.MaxValue) System.Array.Resize (ref data, byte.MaxValue);
            writeByte (data.Length);
            writeBytes (data);
        }
        public void writeMultiByte (string text, string encoding = "utf-8") {
            if (text == "" || text == null) return;

            byte[] data;
            if (encoding == "utf-8") {
                data = System.Text.Encoding.UTF8.GetBytes (text);
            } else {
                data = System.Text.Encoding.ASCII.GetBytes (text);
            }
            writeBytes (data);
        }
        public void writeBytes (byte[] data) {
            int ARMX = data.Length;
            for (int i = 0; i < ARMX; i++) {
                writeByte (data[i]);
            }
        }
        public void writeBytesReverse (byte[] data) {
            for (int i = data.Length - 1; i >= 0; i--) {
                writeByte (data[i]);
            }
        }
        public void writeBytes (ByteArray Src, int src_start = 0, int src_length = -1) {
            if (src_length < 0) src_length = ((int) Src.Length - src_start);
            src_length += src_start;
            for (int i = src_start; i < src_length; i++) {
                writeByte (Src.bytes[i]);
            }
        }
        public BinaryWriter writeToFileStream (FileStream Fs, BinaryWriter Wr) {
            if (Wr == null) Wr = new BinaryWriter (Fs);
            Wr.Write (bytes, 0, (int) position);
            return Wr;
        }
        public void writeDictionary (Dictionary<string, int> O) {
            var Akeys = X.objKeys (O);
            int ARMX = Akeys.Length;
            writeUShort ((ushort) ARMX);
            string k;
            for (int i = 0; i < ARMX; i++) {
                writeString (k = Akeys[i]);
                writeInt (O[Akeys[i]]);
            }
        }
        public void writeDictionary (Dictionary<string, byte> O) {
            var Akeys = X.objKeys (O);
            int ARMX = Akeys.Length;
            writeUShort ((ushort) ARMX);
            string k;
            for (int i = 0; i < ARMX; i++) {
                writeString (k = Akeys[i]);
                writeByte (O[Akeys[i]]);
            }
        }
        public void writeDictionaryT<T> (Dictionary<T, float> O, Func<T, int> FnConvertT2Int) {
            writeUShort ((ushort) O.Count);
            foreach (KeyValuePair<T, float> _kvp in O) {
                writeInt (FnConvertT2Int (_kvp.Key));
                writeFloat (_kvp.Value);
            }
        }
        public void writeDictionaryT<T> (Dictionary<T, int> O, Func<T, int> FnConvertT2Int) {
            writeUShort ((ushort) O.Count);
            foreach (KeyValuePair<T, int> _kvp in O) {
                writeInt (FnConvertT2Int (_kvp.Key));
                writeInt (_kvp.Value);
            }
        }
        public ByteArray writeExtractBytes (ByteArray srcBa, int len_count = 4, int read_len = -1) {
            if (read_len < 0) read_len = (int) srcBa.Length;
            if (len_count == 4) writeUInt ((uint) read_len);
            else if (len_count == 2) writeUShort ((ushort) read_len);
            else writeByte ((byte) read_len);
            writeBytes (srcBa, 0, read_len);
            return srcBa;
        }
        public ByteArray writeExtractBytesShifted (ByteArray srcBa, byte sh = 73, int len_count = 4, int read_len = -1) {
            if (read_len < 0) read_len = (int) srcBa.Length;
            if (len_count == 4) writeUInt ((uint) read_len);
            else if (len_count == 2) writeUShort ((ushort) read_len);
            else writeByte ((byte) read_len);
            writeByte (sh);
            for (int i = 0; i < read_len; i++) {
                writeByte ((srcBa.bytes[i] + sh) & 0xff);
            }
            return srcBa;
        }
        public ByteArray SeekSet () { position = 0; return this; }
        // public void writeDictionary (Dictionary<string, float> O) {
        //     var Akeys = X.objKeys (O);
        //     int ARMX = Akeys.Length;
        //     writeUShort ((ushort) ARMX);
        //     string k;
        //     for (int i = 0; i < ARMX; i++) {
        //         writeString (k = Akeys[i]);
        //         writeFloat (O[Akeys[i]]);
        //     }
        // }
    }
}