﻿// ======
// small PixelLiner file (.pxls) Loader
// by cloba.u 2017
// ======
using System.Collections;
using System.Collections.Generic;
using PixelLiner.PixelLinerLib;
using UnityEngine;
namespace PixelLiner {
    public class PxlFrame {
        public PxlSequence pSq;
        public string name = "";
        public PxlLayer[] ALay;
        // public Sprite RenderedImg = null;

        // animator で使用
        public int crf60 = 1; // 表示時間
        public int endf60 = 0;
        public PxlMeshDrawer DrawnMesh;

        public PxlFrame (PxlSequence _pSq, PxlLayer[] _ALay = null) {
            pSq = _pSq;
            ALay = _ALay == null?new PxlLayer[0] : _ALay;
        }
        public int countLayers () {
            return ALay.Length;
        }
        public PxlLayer getLayer (int i) {
            return ALay[i];
        }
        public PxlLayer getLayerByName (string t) {
            int ARMX = ALay.Length;
            for (int i = 0; i < ARMX; i++) {
                if (ALay[i].name == t) return ALay[i];
            }
            return null;

        }
        public PxlLayer[] getLayerArray () {
            return ALay;
        }
        public Texture2D getImageTexture () {
            return ALay[0].Img.get_I ();
        }
        public void Apply () {
            // メッシュの再構築を行います。
            // 保持レイヤーの位置やアルファなどをいじった後、反映させるために
            // この関数を呼ぶ必要があります。
            if (DrawnMesh != null) {
                DrawnMesh.makeMesh (this);
            }

        }
        public Mesh getDrawnMesh (bool refine = false, float offset_pixel_x = 0, float offset_pixel_y = 0) {
            if (DrawnMesh == null) {
                DrawnMesh = new PxlMeshDrawer ();
                refine = true;
            }
            if (refine) {
                DrawnMesh.makeMesh (this, offset_pixel_x, offset_pixel_y);
            } else DrawnMesh.Offset (offset_pixel_x, offset_pixel_y);
            return DrawnMesh.getMesh ();
        }
        public void releaseDrawnMesh () {
            DrawnMesh = null;
        }
        public PxlFrame cpFlip (PxlSequence tSq, bool flip_x = true, bool flip_y = false) {
            int len = countLayers ();
            PxlLayer[] _ALay = new PxlLayer[len];
            PxlFrame F = new PxlFrame (tSq, _ALay);
            F.name = this.name;
            F.crf60 = this.crf60;
            for (int i = 0; i < len; i++) {
                _ALay[i] = ALay[i].cpFlip (F, flip_x, flip_y);
            }
            return F;
        }
        public bool readFromBytes (ByteArray BaBase) {
            ByteArray BaHead = new ByteArray ();
            uint hdlen = BaBase.readUnsignedInt ();
            BaBase.readBytes (BaHead, 0, hdlen);
            int vers = 0;
            try {
                vers = (int) BaHead.readByte ();
                crf60 = (int) BaHead.readShort ();
                name = BaHead.readString ();

                /*  int bt = */
                if (BaHead.bytesAvailable > 0) BaHead.readUByte ();
            } catch (System.Exception err) {
                throw (new System.Exception ("Error in loading PxlFrame " + this.ToString () + ":" + err.Message));
            }

            int llen = BaBase.readShort ();
            System.Array.Resize (ref ALay, llen);
            int li = 0;
            for (int j = 0; j < llen; j++) {
                PxlLayer L = new PxlLayer (this);
                if (L.readFromBytes (BaBase, vers)) {
                    ALay[li++] = L;
                }
            }
            System.Array.Resize (ref ALay, li);
            return li > 0;
        }
        public PxlCharacter pChar {
            get { return pSq != null ? pSq.pChar : null; }
        }
        public PxlPose pPose {
            get { return pSq != null ? pSq.pPose : null; }
        }
        public override string ToString () {
            return (pSq.ToString ()) + ':' + index;
        }
        public int index {
            get { return pSq != null? pSq.IndexOf (this) : 0; }
        }

    }
}