﻿// ======
// small PixelLiner file (.pxls) Loader
// by cloba.u 2017
// ======
using System.Collections;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using PixelLiner.PixelLinerLib;
using UnityEngine;
namespace PixelLiner {
    public class PxlImage {
        public const int TYPE_LUSTER = PxlLayer.TYPE_LUSTER;
        public const int TYPE_VECTOR = PxlLayer.TYPE_VECTOR;
        public const int TYPE_LUSTER_HAS_VECTOR = 8;
        private static uint id2_top = 0;
        public PxlCharacter pChar;
        public int type = TYPE_LUSTER;
        private Texture2D I;
        private Texture2D P; // parts image
        public PxlsImgAtlas AtlasI;
        public PxlsImgAtlas AtlasP;
        public PxlVectorData VecData;
        public Rect RectIUv = new Rect (0, 0, 1, 1);
        public Rect RectPUv = new Rect (0, 0, 1, 1);
        public uint id;
        public double id2;
        public int width = 1;
        public int height = 1;
        // public float scale = 1;
        private List<PxlLayer> ALink;
        public static Regex RegIdstr = new Regex (@"EDI([0-9a-f]+)_(\d+)");

        public static TextureFormat RenderingTextureFormat = TextureFormat.ARGB32;

        public PxlImage (PxlCharacter _pChar, ByteArray SrcBytes = null) {
            pChar = _pChar;
            ALink = new List<PxlLayer> ();
            if (SrcBytes != null) {
                readFromBytes (SrcBytes);
            } else {
                id = (uint) (Random.Range (0, 0xfffff));
                if (id2_top == 0) id2_top = (uint) (1 + Random.Range (0, 0xfffffe));
                id2 = (double) (id2_top++);
            }
        }
        public PxlImage (PxlCharacter _pChar, string key) {
            pChar = _pChar;
            Match Nobj = RegIdstr.Match (key);
            ALink = new List<PxlLayer> ();
            if (Nobj.Success) {
                id = uint.Parse (Nobj.Groups[1].Value, System.Globalization.NumberStyles.AllowHexSpecifier);
                id2 = double.Parse (Nobj.Groups[2].Value);
                return;
            } else {
                id = (uint) (Random.Range (0, 0xfffff));
                if (id2_top == 0) id2_top = (uint) (1 + Random.Range (0, 0xfffffe));
                id2 = (double) (id2_top++);
            }

        }
        public void readFromBytes (ByteArray Cd) {
            type = Cd.readByte () - 22;
            if (type < 0) return;

            /* int vers =*/
            int vers = Cd.readByte ();
            id = Cd.readUnsignedInt ();
            id2 = Cd.readDouble ();

            uint len;
            if (type == TYPE_LUSTER || type == TYPE_LUSTER_HAS_VECTOR) {
                // normal image
                len = Cd.readUnsignedInt ();
                if (len != 0) {
                    ByteArray BufI = new ByteArray ();
                    Cd.readBytes (BufI, 0, len);
                    I = createFromPngRawData (BufI);
                    RectIUv.Set (0, 0, 1, 1);
                    //pChar.loading_image_progress_count++;
                }
                len = Cd.readUnsignedInt ();
                if (len != 0) {
                    ByteArray BufP = new ByteArray ();;
                    Cd.readBytes (BufP, 0, len);
                    P = createFromPngRawData (BufP);
                    RectPUv.Set (0, 0, 1, 1);

                    // pChar.loading_image_progress_count++;
                }
            }
            if (valid) pChar.assignImage (this);

        }
        public bool valid {
            get { return (type == TYPE_LUSTER || type == TYPE_LUSTER_HAS_VECTOR) ? (I != null || P != null) : false; }
        }
        public string idstr {
            get { return "EDI" + id.ToString ("x") + "_" + id2.ToString (); }
        }
        public static string getIdString (uint id, double id2) {
            string idstr = "EDI" + id.ToString ("x") + "_" + id2;
            return idstr;
        }
        public int get_type () {
            return type;
        }
        public Texture2D get_I () {
            return (I != null) ? I : (AtlasI != null?AtlasI.getImage () : null);
        }
        public bool get_input_wh (ref int w, ref int h) {
            if (AtlasI != null) {
                w = AtlasI.input_width;
                h = AtlasI.input_height;
            } else {
                var I = get_I ();
                if (I != null) {
                    w = I.width;
                    h = I.height;
                } else return false;
            }
            return true;

        }
        public PxlImage set_I (PxlsImgAtlas Atlas, Rect Rc) {
            float w = Atlas.input_width,
                h = Atlas.input_height;
            AtlasI = Atlas;
            width = (int) Rc.width;
            height = (int) Rc.height;
            RectIUv.Set (Rc.x / w, Rc.y / h, Rc.width / w, Rc.height / h);
            return this;
        }
        public Texture2D get_P () {
            return (P != null) ? P : (AtlasP != null?AtlasP.getImage () : null);
        }
        public PxlImage set_P (PxlsImgAtlas Atlas, Rect Rc) {
            float w = Atlas.input_width,
                h = Atlas.input_height;
            AtlasP = Atlas;
            RectPUv.Set (Rc.x / w, Rc.y / h, Rc.width / w, Rc.height / h);
            return this;
        }
        public float draw_scale {
            get {
                if (AtlasI != null) {
                    return AtlasI.getDrawScale ();
                }
                return 1;
            }
        }
        public PxlLayer getImportSource (PxlLayer L) {

            PxlCharacter Csp = L.pChar;
            int ARMX = ALink.Count;
            for (int i = 0; i < ARMX; i++) {
                if (ALink[i].isRelevance ()) continue;
                if (ALink[i] == L) continue;
                if (ALink[i].pChar != Csp) continue;
                return ALink[i];
            }
            return null;
        }
        public PxlImage setVectorData (PxlVectorData _Vc) {
            if (VecData != null) {
                if (VecData == _Vc) return this;
                VecData = null;
            }
            VecData = _Vc;
            if (VecData != null && VecData.isLusterLayerMask ()) {
                if (type == TYPE_LUSTER_HAS_VECTOR || type == TYPE_LUSTER) type = TYPE_LUSTER_HAS_VECTOR;
                else {
                    VecData = null;
                }
            }
            return this;
        }
        public PxlImage assign (PxlLayer L) {
            ALink.Add (L);
            return this;
        }
        public PxlImage deassign (PxlLayer L) {
            ALink.Remove (L);
            return this;
        }
        public void Destroy () {
            //if (I) Destroy(I);
            //if (P) Destroy(P);
            VecData = null;
            I = P = null;
        }

        // ======================================================== 
        // MARK: inner 
        // ========================================================
        /*private function imgLoadComplete(ev: Event): void {
            var _Info: LoaderInfo;
            var _Ld: Loader;
            if (ev.currentTarget == InfoI) {
                _Info = InfoI;
                _Ld = LdI;
            } else if (ev.currentTarget == InfoP) {
                _Info = InfoP;
                _Ld = LdP;
            } else {
                if (ev.currentTarget is LoaderInfo) infoUnload(ev.currentTarget as LoaderInfo);
                return;
            }
            var Img: BitmapData = new BitmapData(_Ld.width, _Ld.height, true, 0x00000000);
            Img.draw(_Ld, new Matrix());
            if (_Info == InfoI) I = Img;
            else P = Img;
            infoUnload(_Info);
        }*/
        public static Texture2D createFromPngRawData (ByteArray Ba) {
            // IHDR から 幅と高さを取得
            Ba.position = 0;
            if (Ba.readUnsignedInt () != 0x89504E47) return null;
            if (Ba.readUnsignedInt () != 0x0D0A1A0A) return null;
            Ba.readUnsignedInt ();
            string chunkh = Ba.readMultiByte (4);
            if (chunkh != "IHDR") return null;
            uint width = Ba.readUnsignedInt ();
            uint height = Ba.readUnsignedInt ();

            Ba.position = 0;
            Texture2D Tx = new Texture2D ((int) width, (int) height, RenderingTextureFormat, false, false);
            Tx.filterMode = FilterMode.Point;
            ImageConversion.LoadImage (Tx, Ba.bytes);
            Tx.Apply (false, PxlsLoader.texture_unreadable);
            return Tx;
            //return Sprite.Create(Tx, new Rect(0, 0, width, height), Vector2.zero);
        }
    }
}