// ======
// small PixelLiner file (.pxls) Loader
// by cloba.u 2017
// ======
using System.Collections;
using System.Collections.Generic;
using PixelLiner.PixelLinerLib;
using UnityEngine;
namespace PixelLiner {
    public class PxlLayer {
        public const int TYPE_LUSTER = 0;
        public const int TYPE_IMPORT = 1;
        public const int TYPE_VECTOR = 2;
        public const int TYPE_VIMPORT = 3;
        public const int TYPE_FLOAT = 4;
        public const int TYPE_FIMPORT = 5;
        public const int TYPE_VFLOAT = 6;
        public const int TYPE_VFIMPORT = 7;
        public const int TYPE_GROUP = 8;
        public PxlFrame pFrm;
        PxlImage Img_;
        public int type = TYPE_LUSTER;
        public string name = "Layer";

        public float alpha = 100;
        public float x = 0;
        public float y = 0;
        public float zmx = 1;
        public float zmy = 1;
        public float rotR = 0;
        public int blend_variable;
        public Sprite CreatedSprite;

        public PxlLayer (PxlFrame _pFrm) {
            pFrm = _pFrm;
        }
        public bool readFromBytes (ByteArray Ba, int vers) {
            uint id = Ba.readUnsignedInt ();
            double id2 = (double) Ba.readDouble ();
            type = (int) Ba.readByte ();
            Img = pChar.getImage (PxlImage.getIdString (id, id2));

            name = Ba.readString ();
            alpha = X.MMX (0, (int) (Ba.readShort () / 100), 100);
            x = Ba.readShort () / 10.0f;
            y = Ba.readShort () / 10.0f;
            zmx = (float) Ba.readDouble ();
            zmy = (float) Ba.readDouble ();
            rotR = (float) Ba.readDouble ();

            blend_variable = Ba.readUShort ();
            Ba.readUnsignedInt (); // frame color
            Ba.readByte (); // frame alpha
            Ba.readByte (); // frame width

            if (Img == null) {
                string _error = "Unknown Image for:" + this.ToString () + " ( key:" + PxlImage.getIdString (id, id2) + " ) ";
                Debug.Log (_error);
                //  throw (new System.Exception ();
            }

            return Img != null;
        }
        public PxlLayer cpFlip (PxlFrame tF, bool flip_x = true, bool flip_y = false) {
            PxlLayer Lay = new PxlLayer (tF);
            Lay.Img = this.Img;
            Lay.type = this.type;
            Lay.name = this.name;
            Lay.alpha = this.alpha;
            Lay.x = (flip_x? - 1 : 1) * this.x;
            Lay.y = (flip_y? - 1 : 1) * this.y;
            Lay.zmx = (flip_x? - 1 : 1) * this.zmx;
            Lay.zmy = (flip_y? - 1 : 1) * this.zmy;
            Lay.rotR = this.rotR;
            //if (flip_x) Lay.rotR = X.PI - Lay.rotR;
            if ((flip_x? - 1 : 1) * (flip_y? - 1 : 1) < 0) Lay.rotR = -Lay.rotR;
            Lay.blend_variable = this.blend_variable;
            return Lay;
        }
        public PxlImage Img {
            get { return Img_; }
            set {
                if (Img_ != null) Img_.deassign (this);
                Img_ = value;
                if (Img_ != null) Img_.assign (this);
            }
        }
        public bool isVector () {
            return (type & 2) > 0 ? true : false;
        }
        public bool isImport () {
            return (type & 1) > 0 ? true : false;
        }
        public bool isDrawable () {
            return !isImport () && !isVector () && !isGroup ();
        }
        public bool isFloat () {
            return (type & 4) > 0 ? true : false;
        }
        public bool isGroup () {
            return (type & 8) > 0 ? true : false;
        }
        public bool isRelevance () { // 単独で情報をもっていないレイヤー
            return isImport () || isGroup ();
        }
        public PxlLayer getImportSource () {
            return isImport () ? Img.getImportSource (this) : this;
        }
        public Sprite createImageSprite (int margin = 0) {
            return createImageSpriteInner (new Vector2 (0.5f, 0.5f), margin, new Vector4 ());
        }
        public Sprite createImageSprite (Vector2 Pivot, int margin = 0, uint extrude = 0, SpriteMeshType meshType = SpriteMeshType.FullRect) {
            return createImageSpriteInner (Pivot, margin, new Vector4 (), extrude, meshType);
        }
        public Sprite createImageSprite (Vector2 Pivot, int margin, uint extrude, SpriteMeshType meshType, Vector4 border) {
            return createImageSpriteInner (Pivot, margin, border, extrude, meshType, true);
        }
        public Sprite createImageSprite (Vector2 Pivot, int margin, uint extrude, SpriteMeshType meshType, Vector4 border, bool generateFallbackPhysicsShape) {
            return createImageSpriteInner (Pivot, margin, border, extrude, meshType, true, generateFallbackPhysicsShape);
        }
        private Sprite createImageSpriteInner (Vector2 Pivot, int margin, Vector4 border, uint extrude = 0, SpriteMeshType meshType = SpriteMeshType.FullRect, //
            bool use_border = false, bool generateFallbackPhysicsShape = false) {
            Rect Rc = Img.RectIUv;
            var I = Img.get_I ();
            Rc.x = Rc.x * I.width - margin;
            Rc.width = Rc.width * I.width + margin * 2;
            Rc.y = Rc.y * I.height - margin;
            Rc.height = Rc.height * I.height + margin * 2;
            if (use_border) {
                CreatedSprite = Sprite.Create (I, Rc, Pivot, pChar.pixelsPerUnit / Img.draw_scale, extrude, meshType, border, generateFallbackPhysicsShape);
            } else {
                CreatedSprite = Sprite.Create (I, Rc, Pivot, pChar.pixelsPerUnit / Img.draw_scale, extrude, meshType);
            }
            return CreatedSprite;
        }
        public Mesh makeMesh (float shift_pixel_x, float shift_pixel_y, Mesh Dest = null, bool append_after = false) {
            var Rc = Img.RectIUv;
            float shiftx = shift_pixel_x / pChar.pixelsPerUnit,
                shifty = shift_pixel_y / pChar.pixelsPerUnit,
                wh = (Img.width / 2) / pChar.pixelsPerUnit,
                hh = (Img.height / 2) / pChar.pixelsPerUnit;
            int tri0 = append_after && Dest != null ? Dest.vertexCount : 0;
            var Atri = new int[6] { tri0 + 0, tri0 + 1, tri0 + 2, tri0 + 0, tri0 + 2, tri0 + 3 };
            var AUv = new Vector2[4] {
                new Vector2 (Rc.x, Rc.y),
                new Vector2 (Rc.x, Rc.y + Rc.height),
                new Vector2 (Rc.x + Rc.width, Rc.y + Rc.height),
                new Vector2 (Rc.x + Rc.width, Rc.y)
            };
            var AVer = new Vector3[4] {
                new Vector3 (shiftx - wh, shifty - hh, 0),
                new Vector3 (shiftx - wh, shifty + hh, 0),
                new Vector3 (shiftx + wh, shifty + hh, 0),
                new Vector3 (shiftx + wh, shifty - hh, 0),
            };
            if (Dest == null) {
                Dest = new Mesh ();
                Dest.vertices = AVer;
                Dest.triangles = Atri;
                Dest.uv = AUv;
            } else {
                if (!append_after) {
                    X.pushA (ref AVer, Dest.vertices);
                    X.pushA (ref AUv, Dest.uv);
                    X.pushA (ref Atri, Dest.triangles);
                    int ARMX = Atri.Length;
                    for (int i = 6; i < ARMX; i++) {
                        Atri[i] += 4;
                    }
                    Dest.vertices = AVer;
                    Dest.triangles = Atri;
                    Dest.uv = AUv;
                } else {
                    Vector3[] ADestVer = Dest.vertices;
                    Vector2[] ADestUv = Dest.uv;
                    int[] Atriangle = Dest.triangles;
                    X.pushA (ref ADestVer, AVer);
                    X.pushA (ref ADestUv, AUv);
                    X.pushA (ref Atriangle, Atri);
                    Dest.vertices = ADestVer;
                    Dest.uv = AUv;
                    Dest.triangles = Atriangle;
                }
            }
            return Dest;
        }

        public int lblend {
            get {
                return (blend_variable) & 0xff;
            }
            set {
                blend_variable = ((blend_variable) & (~0xff)) | (value & 0xff);
            }
        }
        public int ablend {
            get {
                return (blend_variable >> 8) & 0xff;
            }
            set {
                blend_variable = ((blend_variable) & (~0xff00)) | ((value & 0xff) << 8);
            }
        }
        public Matrix4x4 getTransformMatrix () {
            return Matrix4x4.Translate (new Vector3 (x / pChar.pixelsPerUnit, -y / pChar.pixelsPerUnit, 0)) * Matrix4x4.Rotate (Quaternion.Euler (0, 0, -rotR / X.PI * 180)) * Matrix4x4.Scale (new Vector3 (zmx, zmy, 1));
        }
        public PxlCharacter pChar {
            get { return pFrm != null ? pFrm.pChar : null; }
        }
        public int cvs_left {
            get {
                // カンバス左基点での左位置
                return (int) x - (int) (Img.width / 2) + (int) (pFrm.pPose.width / 2);
            }
        }
        public int cvs_top {
            get {
                // カンバス左基点での左位置
                return (int) y - (int) (Img.height / 2) + (int) (pFrm.pPose.height / 2);
            }
        }
        public override string ToString () {
            return "<PxlLayer>" + name + " ... " + pFrm.ToString ();
        }

    }
}