﻿using System.Collections;
using System.Collections.Generic;
using PixelLiner.PixelLinerLib;
using UnityEngine;

namespace PixelLiner {
    public class PxlMeshDrawer {
        public PxlFrame SourceFrame;
        private float offsetx = 0;
        private float offsety = 0;
        public static bool do_not_draw_hidden_layer = false; // アルファが 0 のレイヤーを描かない

        public PxlMeshDrawer () {
            Ms = new Mesh ();
        }
        public Mesh makeMesh (PxlFrame F) {
            return makeMesh (F, offsetx, offsety);
        }
        public Mesh makeMesh (PxlFrame F, float offset_pixel_x, float offset_pixel_y) {
            SourceFrame = F;
            int fcnt = F.countLayers ();
            int fcnt4 = fcnt * 4,
                fcnt6 = fcnt * 6;
            Col.r = Col.g = Col.b = Col.a = 255;

            if (AVertice == null || AVertice.Length != fcnt4) {
                AVertice = new Vector3[fcnt4];
                AMeshUv = new Vector2[fcnt4];
                Acolor = new Color32[fcnt4];
                Atriangle = new int[fcnt6];
            }

            ver_i = 0;
            tri_i = 0;
            //ppu = F.pChar.pixelsPerUnit;
            int inputw = 0, inputh = 0;
            for (int i = 0; i < fcnt; i++) {
                PxlLayer L = F.getLayer (i);
                if (i == 0) {
                    L.Img.get_input_wh (ref inputw, ref inputh);
                }
                if (inputw <= 0) continue;
                RectUv = L.Img.RectIUv;

                if (do_not_draw_hidden_layer && L.alpha <= 0) continue;
                Col.a = (byte) (L.alpha * 255 / 100);
                DrawRotaGraph (L.x, -L.y, L.zmx, L.zmy, -L.rotR, inputw, inputh);
            }
            if (ver_i != AVertice.Length) {
                System.Array.Resize (ref AVertice, ver_i);
                System.Array.Resize (ref AMeshUv, ver_i);
                System.Array.Resize (ref Acolor, ver_i);
            }
            if (tri_i != Atriangle.Length) {
                System.Array.Resize (ref Atriangle, tri_i);
            }
            Ms.Clear ();
            Ms.vertices = AVertice;
            Ms.uv = AMeshUv;
            Ms.colors32 = Acolor;
            Ms.triangles = Atriangle;

            if (offset_pixel_x != offsetx || offset_pixel_y != offsety) {
                Offset (offset_pixel_x, offset_pixel_y);
            } else {
                Ms.RecalculateNormals ();
                Ms.RecalculateBounds ();
            }
            return Ms;
        }
        public PxlMeshDrawer Offset (float x, float y) {
            // ピクセルで指定する
            if (x == offsetx && y == offsety) return this;
            float ppu = SourceFrame.pChar.pixelsPerUnit;
            Vector3 Dif = new Vector3 ((x - offsetx) / ppu, (y - offsety) / ppu, 0);
            offsetx = x;
            offsety = y;
            for (int i = 0; i < ver_i; i++) {
                AVertice[i] = AVertice[i] + Dif;
            }
            Ms.vertices = AVertice;
            Bounds bounds = Ms.bounds;
            bounds.center = bounds.center + Dif;
            Ms.bounds = bounds;
            return this;
        }

        public Color32 Col;
        int ver_i = 0;
        int tri_i = 0;
        Vector2[] AMeshUv;
        Color32[] Acolor;
        Vector3[] AVertice;
        int[] Atriangle;
        Mesh Ms;

        public Rect RectUv; // ゲーム座標

        public PxlMeshDrawer Tri (int t0, int t1, int t2) {
            if (tri_i + 2 >= Atriangle.Length) {
                System.Array.Resize (ref Atriangle, tri_i + 18);
            }
            Atriangle[tri_i++] = ver_i + t0;
            Atriangle[tri_i++] = ver_i + t1;
            Atriangle[tri_i++] = ver_i + t2;
            return this;
        }
        public PxlMeshDrawer Pos (float x, float y, Matrix4x4 Mtrx) {
            if (ver_i >= AVertice.Length) {
                System.Array.Resize (ref AVertice, AVertice.Length + 64);
                System.Array.Resize (ref AMeshUv, AMeshUv.Length + 64);
                System.Array.Resize (ref Acolor, Acolor.Length + 64);
            }
            int vi = ver_i & 3;
            AMeshUv[ver_i].Set (vi == 0 || vi == 1 ? (RectUv.x) : (RectUv.x + RectUv.width), vi == 0 || vi == 3 ? (RectUv.y) : (RectUv.y + RectUv.height)); //
            { //
                Vector3 T = new Vector3 (x, y, 0);
                T = Mtrx.MultiplyPoint3x4 (T);
                x = T.x;
                y = T.y;
            }
            AVertice[ver_i].Set (x, y, 0);
            Acolor[ver_i++] = (Col);
            return this;
        }
        public PxlMeshDrawer RectBL (float x, float y, float w, float h, Matrix4x4 Mtrx, bool no_divide_ppu = false, bool flipped = false) {
            if (!no_divide_ppu) {
                float ppu = SourceFrame.pChar.pixelsPerUnit;
                x /= ppu;
                y /= ppu;
                w /= ppu;
                h /= ppu;
            }
            if (flipped) {
                Tri (0, 2, 1).Tri (0, 3, 2);
            } else Tri (0, 1, 2).Tri (0, 2, 3);
            float b = y + h,
                r = x + w;
            Pos (x, y, Mtrx).Pos (x, b, Mtrx).Pos (r, b, Mtrx).Pos (r, y, Mtrx);
            return this;
        }
        // ======================================================== 
        // MARK: for matrix 
        // ========================================================
        private PxlMeshDrawer DrawRotaGraph (float x, float y, float scalex = 1, float scaley = 1, float rotR = 0, int sourcew = 0, int sourceh = 0) {
            // pivot_x : 0 なら画像左基点、 1 なら画像右基点
            float w, h;
            Matrix4x4 Mtrx = Matrix4x4.identity;
            if (scalex == 1 && scaley == 1 && rotR == 0) {
                w = Mathf.Round (sourcew * RectUv.width);
                h = Mathf.Round (sourceh * RectUv.height);
                return RectBL (x - (int) (w * 0.5f), y + (int) (h * 0.5f) - h, w, h, Mtrx);
            } else {
                w = sourcew * RectUv.width * 1;
                h = sourceh * RectUv.height * 1;
                float ppu = SourceFrame.pChar.pixelsPerUnit;
                Mtrx = Matrix4x4.Scale (new Vector3 (scalex, scaley, 1));
                if (rotR != 0) Mtrx = Matrix4x4.Rotate (Quaternion.Euler (0, 0, rotR / X.PI * 180)) * Mtrx;
                Mtrx = Matrix4x4.Translate (new Vector3 (x / ppu, y / ppu)) * Mtrx;
            }
            return RectBL (-(int) (w * 0.5f), (int) (h * (0.5f)) - h, w, h, Mtrx, false, scalex * scaley < 0);
        }
        public Mesh getMesh () {
            return Ms;
        }
        public PxlMeshDrawer releaseCache () {
            AMeshUv = null;
            AVertice = null;
            Acolor = null;
            Atriangle = null;
            return this;
        }
        public void Destroy () {
            SourceFrame = null;
            Ms = null;
        }
        public Texture2D TxUsing {
            get { return SourceFrame.getImageTexture (); }
        }
        public float width {
            get { return SourceFrame != null?(SourceFrame.pPose.width) : 64; }
        }
        public float height {
            get { return SourceFrame != null?(SourceFrame.pPose.height) : 64; }
        }
    }
}