// ======
// small PixelLiner file (.pxls) Loader
// by cloba.u 2017
// ======
using System.Collections;
using System.Collections.Generic;
using PixelLiner.PixelLinerLib;
using UnityEngine;
namespace PixelLiner {
    public class PxlPose {
        public PxlCharacter pChar;

        public string title;
        public int width; // cvw カンバスの大きさに相当します
        public int height; // cvh

        public PxlSequence[] ASq;

        // 以下は非公開機能
        public string[] Aalias_to;
        public bool auto_flip = false;
        public bool tetra_pose = false;
        public string end_jump_title;
        public int end_jump_loop_count;

        public const int AIM_BYTE_CAP = 10;

        public const uint A_L = 0;
        public const uint A_T = 1;
        public const uint A_R = 2;
        public const uint A_B = 3;
        public const uint A_LT = 4;
        public const uint A_TR = 5;
        public const uint A_BL = 6;
        public const uint A_RB = 7;
        public const uint AIM_MAX = 8;

        int flip_maked = 0;
        int copied = 0;
        public string comment = "";
        public PxlPose (PxlCharacter _pChar) {
            pChar = _pChar;
            ASq = new PxlSequence[AIM_MAX];
        }
        public PxlSequence getSequence (int i) {
            return ASq[i];
        }
        public bool readFromBytes (ByteArray Cd) {
            // header section
            uint btlen = Cd.readUnsignedInt ();
            ByteArray BaHead = new ByteArray ();
            Cd.readBytes (BaHead, 0, btlen);
            BaHead.position = 0;

            int j;

            try {
                int vers = (int) BaHead.readByte ();
                auto_flip = BaHead.readBoolean ();
                tetra_pose = BaHead.readBoolean ();
                title = BaHead.readString ();
                width = (int) BaHead.readUShort ();
                height = (int) BaHead.readUShort ();
                end_jump_loop_count = BaHead.readUShort ();
                end_jump_title = BaHead.readString ();

                int ARMXj = (int) BaHead.readUShort ();
                Aalias_to = new string[ARMXj];
                for (j = 0; j < ARMXj; j++) {
                    Aalias_to[j] = BaHead.readString ();
                }
                if (vers >= 2) {
                    comment = BaHead.readString ();
                }

            } catch (System.Exception err) {
                throw (new System.Exception ("Error in loading PxlSeuence " + this.ToString () + ":" + err.Message));
            }
            // body section
            int _a = 0;
            bool valid = false;
            while ((_a = Cd.readByte ()) != 0) {
                _a -= AIM_BYTE_CAP; // aim
                PxlSequence Sq = new PxlSequence (this, _a);
                if (Sq.readFromBytes (Cd) && CAim.valid (_a)) {
                    ASq[_a] = Sq;
                    valid = true;
                }
            }

            if (valid) {
                fillEmptyPose ();
            }
            return valid;
        }

        public string[] getAliasNameArray () {
            return Aalias_to;
        }

        public int fillEmptyPose () {
            if (pChar.autoFlipX) auto_flip = true;
            uint i;
            int first = -1;
            // 自動反転
            for (i = 0; i < AIM_MAX; i++) {
                if (ASq[i] != null) {
                    first = (int) i;
                    break;
                }
            }
            int pcnt = 0;
            // 反転合成
            for (i = 0; i < AIM_MAX; i++) {
                if (ASq[i] == null && auto_flip && (copied & (1 << (byte) i)) == 0) {
                    switch (i) {
                        case A_L:
                            if ((ASq[A_R] != null)) cpFlip (i, ASq[A_R]);
                            else if ((ASq[A_RB] != null)) cpFlip (i, ASq[A_RB]);
                            else if ((ASq[A_TR] != null)) cpFlip (i, ASq[A_TR]);
                            break;
                        case A_R:
                            if ((ASq[A_L] != null)) cpFlip (i, ASq[A_L]);
                            else if ((ASq[A_BL] != null)) cpFlip (i, ASq[A_BL]);
                            else if ((ASq[A_LT] != null)) cpFlip (i, ASq[A_LT]);
                            break;
                        case A_LT:
                            if ((ASq[A_TR] != null)) cpFlip (i, ASq[A_TR]);
                            else if ((ASq[A_R] != null)) cpFlip (i, ASq[A_R]);
                            else if ((ASq[A_RB] != null)) cpFlip (i, ASq[A_RB]);
                            break;
                        case A_TR:
                            if ((ASq[A_LT] != null)) cpFlip (i, ASq[A_LT]);
                            else if ((ASq[A_L] != null)) cpFlip (i, ASq[A_L]);
                            else if ((ASq[A_BL] != null)) cpFlip (i, ASq[A_BL]);
                            break;
                        case A_BL:
                            if ((ASq[A_RB] != null)) cpFlip (i, ASq[A_RB]);
                            else if ((ASq[A_R] != null)) cpFlip (i, ASq[A_R]);
                            else if ((ASq[A_TR] != null)) cpFlip (i, ASq[A_TR]);
                            break;
                        case A_RB:
                            if ((ASq[A_BL] != null)) cpFlip (i, ASq[A_BL]);
                            else if ((ASq[A_L] != null)) cpFlip (i, ASq[A_L]);
                            else if ((ASq[A_LT] != null)) cpFlip (i, ASq[A_LT]);
                            break;
                    }
                }
                pcnt += ASq[i] != null ? 1 : 0;
                /* if (ASq[i]) {
                     ASq[i].defineSoundArray(ASound[i]);
                 }*/
            }
            for (i = 0; i < AIM_MAX; i++) {
                if (ASq[i] == null) {
                    int _yd0 = CAim._YD ((int) i);
                    for (int k = 0; k < 2 && ASq[i] == null; k++) {
                        if (i != (int) AIM.T && i != (int) AIM.B) {
                            for (int j = 0; j < 3; j++) {
                                int _yd;
                                if (j == 1) {
                                    if (_yd0 == 0) _yd = 1;
                                    else _yd = 0;
                                } else if (j == 2) {
                                    if (_yd0 == 0) _yd = -1;
                                    else _yd = -_yd0;
                                } else _yd = _yd0;
                                int a_r = (int) CAim.get_aim_r (0, 0, CAim._XD ((int) i), _yd);
                                if (k == 0 && (!isValidAim (a_r) || isFlipped (a_r))) continue;
                                if (ASq[a_r] != null) {
                                    ASq[i] = ASq[a_r];
                                    copied |= (1 << (byte) i);
                                    break;
                                }
                            }
                        } else {
                            for (int j = 0; j < 2; j++) {
                                int a_r = (int) CAim.get_aim_r (0, 0, j == 0 ? -1 : 1, _yd0);
                                if (k == 0 && (!isValidAim (a_r) || isFlipped (a_r))) continue;
                                if (ASq[a_r] != null) {
                                    ASq[i] = ASq[a_r];
                                    copied |= (1 << (byte) i);
                                    break;
                                }
                            }
                        }
                    }
                    if (ASq[i] == null) { ASq[i] = ASq[first]; copied |= (1 << (byte) i); }
                } // 埋めます
            }
            for (i = 0; i < AIM_MAX; i++) {
                if (ASq[i] == null) { ASq[i] = ASq[first]; copied |= (1 << (byte) i); }
            }
            return pcnt;
        }
        void cpFlip (uint aim_i, PxlSequence Src, bool flip_x = true, bool flip_y = false) {
            flip_maked |= 1 << (byte) aim_i;
            ASq[aim_i] = Src.cpFlip ((int) aim_i, flip_x, flip_y);

        }
        public bool isValidAim (int _a) {
            return ASq[_a] != null && (copied & (1 << (byte) _a)) == 0;
        }
        public int getAim (PxlSequence Sq) {
            for (int i = 0; i < AIM_MAX; i++) {
                if (ASq[i] == Sq) return i;
            }
            return -1;
        }
        public bool isFlipped (int i) {
            return ((flip_maked & (1 << (byte) i)) > 0) ? true : false;
        }

        public int ListupFrames (PxlFrame[] _AFrame, int j, bool include_flip = false) {
            for (int i = 0; i < AIM_MAX; i++) {
                if (isValidAim (i) && (include_flip || !isFlipped (i))) {
                    j = ASq[i].ListupFrames (_AFrame, j);
                }
            }
            return j;
        }
        public int countFrames (bool include_flip = false) {
            int j = 0;
            for (int i = 0; i < AIM_MAX; i++) {
                if (isValidAim (i) && (include_flip || !isFlipped (i))) {
                    j += ASq[i].countFrames ();
                }
            }
            return j;

        }
        public override string ToString () {
            return "[" + pChar.title + "]." + (title);
        }

    }
}