﻿// ======
// small PixelLiner file (.pxls) Loader
// by cloba.u 2017
// ======
using System.Collections;
using System.Collections.Generic;
using PixelLiner.PixelLinerLib;
using UnityEngine;
namespace PixelLiner {
    public class PxlSequence {
        public PxlCharacter pChar;
        public PxlPose pPose;
        public int aim;

        public int width; // 内枠
        public int height; // 内枠
        public PxlFrame[] AFrm;
        public int loop_to = 0;

        // 非公開機能
        public int bodyx = 0;
        public int bodyy = 0;
        public int shiftx = 0;
        public int shifty = 0;
        public string[] Aframe_snd;

        public PxlSequence (PxlPose _pPose, int _aim, PxlFrame[] _AFrame = null) {
            pChar = _pPose.pChar;
            pPose = _pPose;
            aim = _aim;
            AFrm = _AFrame == null?new PxlFrame[0] : _AFrame;
        }
        public int countFrames () {
            return AFrm.Length;
        }
        public PxlFrame getFrame (int i) {
            return AFrm[i];
        }
        public int getDuration () {
            int duration = 0;
            int maisu = countFrames ();
            for (int i = 0; i < maisu; i++) {
                duration += AFrm[i].crf60;
            }
            return duration;

        }
        public PxlFrame getFrameByName (string title) {
            int ARMX = AFrm.Length;
            for (int i = 0; i < ARMX; i++) {
                if (AFrm[i].name == title) return AFrm[i];
            }
            return null;

        }
        public PxlImage getImage (int frame_index, int layer_index = 0) {
            return AFrm[frame_index].getLayer (layer_index).Img;
        }
        public int ListupFrames (PxlFrame[] _AFrame, int j) {
            int maisu = countFrames ();
            for (int i = 0; i < maisu; i++) {
                _AFrame[j++] = AFrm[i];
            }
            return j;
        }
        public PxlSequence cpFlip (int aim, bool flip_x = true, bool flip_y = false) {
            int maisu = countFrames ();
            PxlFrame[] _AFrame = new PxlFrame[maisu];
            PxlSequence tSq = new PxlSequence (pPose, aim, _AFrame);
            tSq.width = this.width;
            tSq.height = this.height;
            tSq.loop_to = this.loop_to;
            tSq.bodyx = (flip_x? - 1 : 1) * this.bodyx;
            tSq.bodyy = (flip_y? - 1 : 1) * this.bodyy;
            tSq.shiftx = (flip_x? - 1 : 1) * this.shiftx;
            tSq.shifty = (flip_y? - 1 : 1) * this.shifty;
            for (int i = 0; i < maisu; i++) {
                _AFrame[i] = AFrm[i].cpFlip (tSq, flip_x, flip_y);
            }
            return tSq;
        }
        public bool readFromBytes (ByteArray BaBase) {
            ByteArray BaHead = new ByteArray ();
            uint hdlen = BaBase.readUnsignedInt ();
            BaBase.readBytes (BaHead, 0, hdlen);
            //int vers;
            try {
                //vers =
                BaHead.readByte ();
                this.width = (int) BaHead.readUShort ();
                this.height = (int) BaHead.readUShort ();
                this.bodyx = (int) BaHead.readShort ();
                this.bodyy = (int) BaHead.readShort ();
                this.shiftx = (int) BaHead.readShort ();
                this.shifty = (int) BaHead.readShort ();
                this.loop_to = (int) BaHead.readShort ();

                int ARMX = BaHead.readUShort ();
                if (ARMX > 0) {
                    this.Aframe_snd = new string[ARMX];
                    for (int i = 0; i < ARMX; i++) {
                        this.Aframe_snd[i] = (BaHead.readString ());
                    }
                };
            } catch (System.Exception err) {
                throw (new System.Exception ("Error in loading PxlSeuence " + this.ToString () + ":" + err.Message));
            }
            int flen = BaBase.readUShort ();
            System.Array.Resize (ref AFrm, flen);
            int fi = 0;
            for (int j = 0; j < flen; j++) {
                PxlFrame F = AFrm[fi] = new PxlFrame (this);
                if (F.readFromBytes (BaBase)) {
                    fi++;
                    pChar.frame_count++;
                }
            }
            flen = fi;
            System.Array.Resize (ref AFrm, fi);
            if (this.Aframe_snd != null) System.Array.Resize (ref Aframe_snd, flen);
            loop_to = X.Mn (flen - 1, loop_to);

            return flen > 0;
        }
        public int IndexOf (PxlFrame F) {
            int ARMX = countFrames ();
            for (int i = 0; i < ARMX; i++) {
                if (AFrm[i] == F) return i;
            }
            return -1;
        }
        public override string ToString () {
            return '.' + (pPose.title) + '@' + aim;
        }

    }
}