﻿// ======
// small PixelLiner file (.pxls) Loader
// by cloba.u 2017
// ======
using System.Collections;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using PixelLiner.PixelLinerLib;
using UnityEngine;
namespace PixelLiner {
    public class PxlVectorData {
        public static readonly int TYPE_LUSTER = PxlLayer.TYPE_LUSTER;
        public static readonly int TYPE_VECTOR = PxlLayer.TYPE_VECTOR;
        public static readonly int TYPE_LUSTER_HAS_VECTOR = PxlImage.TYPE_LUSTER_HAS_VECTOR;
        public int type = 0;
        private PxlImage Img;
        public Vector3[] APts;
        public PxlVectorData (PxlImage _Img = null) {
            Img = _Img;

        }
        public bool isLusterLayerMask () {
            return type == TYPE_LUSTER_HAS_VECTOR;
        }
        public void readFromBytes (ByteArray Ba) {
            type = Ba.readByte ();
            int vers = Ba.readByte ();
            uint ARMX = Ba.readUnsignedInt ();
            if (APts == null) {
                APts = new Vector3[ARMX];
            } else System.Array.Resize (ref APts, (int) ARMX);
            for (uint i = 0; i < ARMX; i++) {
                APts[i].Set (Ba.readFloat (), Ba.readFloat (), Ba.readFloat ());
            }
        }
        public static bool progressVectorDataReading (ByteArray Ba, PxlCharacter Pc) {
            // 終わったら false
            // 最初の個数 v_cnt は最初から読み飛ばされています
            uint id = Ba.readUnsignedInt ();
            double id2 = Ba.readDouble ();
            var idstr = PxlImage.getIdString (id, id2);
            PxlImage Img = Pc.getImage (idstr);
            if (Img == null) {
                Img = new PxlImage (Pc, idstr);
                Img.type = PxlImage.TYPE_VECTOR;
                Pc.assignImage (Img);
            }
            // ベクター内部の読み込み
            PxlVectorData Vc = new PxlVectorData (Img);
            Vc.readFromBytes (Ba);
            Img.setVectorData (Vc);
            return Ba.bytesAvailable > 0;
        }
        public void makeCollider (PolygonCollider2D Cld, float pixels_per_unit) {
            // PolygonCollider に ベクターマスクからパスを作成します。
            // APts の各ポイントには画像の中心から見たピクセルが入っています。
            // z の値は 0 または 1　であり、 z が 1 の点は直前の点とつながっていないことを意味します。
            // APts[0] の z は必ず 1　が入ります。
            // なお、パスの点群は時計回りで生成されています。
            if (APts == null) {
                Cld.pathCount = 0;
                return;
            }
            int path_count = 1;
            int maxi = APts.Length;
            for (int i = 1; i < maxi; i++) {
                if (((int) (APts[i].z) & 1) != 0) {
                    // 直前の点とつながっていない
                    path_count++;
                }
            }
            Cld.pathCount = path_count;
            int pre_block_start_i = 0;
            path_count = 0;
            for (int i = 1; i <= maxi; i++) {
                if (i == maxi || ((int) (APts[i].z) & 1) != 0) {
                    // 直前の点とつながっていないか、配列の終わり
                    Vector2[] BlockPts = new Vector2[i - pre_block_start_i];
                    for (int j = pre_block_start_i; j < i; j++) {
                        Vector2 Pos = APts[j] / pixels_per_unit;
                        Pos.y *= -1;
                        BlockPts[j - pre_block_start_i] = Pos;
                    }
                    Cld.SetPath (path_count++, BlockPts);
                    pre_block_start_i = i;
                }
            }
        }
    }
}