﻿using System.Collections;
using System.Collections.Generic;
using PixelLiner.PixelLinerLib;
using UnityEngine;
/*
ゲーム用スプライトファイル作成
スプライトを BLF パッキングした1枚の画像
 */
namespace PixelLiner {
    public class PxlsTexture {
        public string texture_path;
        public Texture2D Image;
        int input_width_;
        int input_height_;
        int source_width;
        int source_height;
        public float scale = 1;
        public void Unload (bool only_external = false) {
            if (Image == null || (only_external && texture_path == null)) return;
            if (input_width_ == 0 && source_width != 0) {
                input_width_ = source_width;
                input_height_ = source_height;
            }
            try {
                Resources.UnloadAsset (Image);
            } catch { }
            Image = null;
        }
        public int width {
            get { return source_width != 0 ? source_width : input_width_; }
        }
        public int height {
            get { return source_height != 0 ? source_height : input_height_; }
        }
        public int input_width {
            get { return input_width_ != 0 ? input_width_ : source_width; }
        }
        public int input_height {
            get { return input_height_ != 0 ? input_height_ : source_height; }
        }
        public PxlsTexture (Texture2D Tx) {
            assignImage (Tx);
        }
        public PxlsTexture (string _texture_path, bool load_execute = true) {
            texture_path = _texture_path;
            if (load_execute) {
                assignImage (Resources.Load<Texture2D> (texture_path));
            }
        }
        public PxlsTexture assignSizeFromBinary (int _iw, int _ih) {
            input_width_ = _iw;
            input_height_ = _ih;
            if (source_width != 0 && source_height != 0) {
                scale = (float) input_width_ / source_width;
            }
            return this;
        }
        public Texture2D assignImage (Texture2D _Image) {
            if (_Image != null) {
                Image = _Image;
                source_width = Image.width;
                source_height = Image.height;
                if (input_width_ != 0 && input_height_ != 0) {
                    scale = (float) input_width_ / source_width;
                }
            }
            return Image;
        }
        public static string PxlsTexturePath (string pxls_path, int num) {
            return pxls_path + ".bytes.texture_" + (num >= 0 ? num + "" : "");
        }
        public static PxlsTexture[] ConvertTextureArray (Texture2D[] ATx) {
            if (ATx == null) return null;
            int ARMX = ATx.Length;
            var A = new PxlsTexture[ARMX];
            for (int i = 0; i < ARMX; i++) {
                A[i] = new PxlsTexture (ATx[i]);
            }
            return A;
        }

    }

    public class PxlsImgAtlas {
        public const int IMG_TYPE_NORMAL = 0;
        public const int IMG_TYPE_PARTS = 1;
        public int img_type = IMG_TYPE_NORMAL;
        internal int margin;
        public int image_index = -1;
        PxlsTexture Image;
        internal PxlsImgAtlasUv[] Apos;
        public PxlCharacter pChar;

        private static PxlsImgAtlas[] AStackedLoding;

        public PxlsImgAtlas () { }
        public void clearInputted () {
            Apos = new PxlsImgAtlasUv[0];
        }

        public void readFromBytes (ByteArray Ba, PxlCharacter _pChar = null) {
            dispose ();
            img_type = Ba.readByte () - 22;
            pChar = _pChar;
            if (img_type >= 0) {
                int vers = Ba.readUByte (); // vers
                margin = Ba.readUByte ();
                uint ARMX = Ba.readUnsignedInt ();
                Apos = new PxlsImgAtlasUv[ARMX];
                for (int i = 0; i < ARMX; i++) {
                    uint k_id = Ba.readUnsignedInt ();
                    double k_id2 = Ba.readDouble ();
                    PxlsImgAtlasUv Da = Apos[i] = new PxlsImgAtlasUv (k_id, k_id2);
                    Da.x = (int) Ba.readUnsignedInt (); // vers
                    Da.y = (int) Ba.readUnsignedInt (); // vers
                    Da.width = (int) Ba.readUnsignedInt (); // vers
                    Da.height = (int) Ba.readUnsignedInt (); // vers
                }
                if (vers == 1) {
                    // read png outputted
                    var dwidth = (int) Ba.readUnsignedInt (); // vers
                    var dheight = (int) Ba.readUnsignedInt (); // vers
                    Image = pChar.loadExternalPngResource (this, dwidth, dheight);
                } else {
                    uint len = Ba.readUnsignedInt ();
                    ByteArray BufI = new ByteArray ();
                    Ba.readBytes (BufI, 0, len);
                    Texture2D Bm = PxlImage.createFromPngRawData (BufI);
                    Image = new PxlsTexture (Bm);
                }
                assignPxlImage ();
            }
        }
        public void assignPxlImage () {
            // read したあとに読み込む
            if (Apos.Length == 0 || pChar == null || Image == null) return;
            int ARMX = Apos.Length;
            Rect Rc = new Rect ();
            for (int i = 0; i < ARMX; i++) {
                PxlsImgAtlasUv Da = Apos[i];
                PxlImage PImg = pChar.getImage (Da.key);
                if (PImg == null) {
                    PImg = new PxlImage (pChar, Da.key);
                    if (img_type == IMG_TYPE_PARTS ? PImg.get_P () : PImg.get_I ()) {
                        Debug.Log ("イメージ指定の重複:" + Da.key);
                    }
                    pChar.assignImage (PImg);
                }

                Rc.x = Da.x + margin;
                Rc.y = Da.y + margin;
                Rc.width = Da.width - margin * 2;
                Rc.height = Da.height - margin * 2;
                // 上下反転
                Rc.y = (Image.input_height) - (Rc.y + Rc.height);
                if (img_type == IMG_TYPE_PARTS) PImg.set_P (this, Rc); //.setPRectByPixel (Rc, dwidth, dheight);
                else PImg.set_I (this, Rc); //.setIRectByPixel (Rc, dwidth, dheight);
            }

        }
        public static void readFromBytesStack (ByteArray Ba, PxlCharacter _pChar = null) {
            if (AStackedLoding == null) AStackedLoding = new PxlsImgAtlas[0];
            PxlsImgAtlas Atlas = new PxlsImgAtlas ();
            int len = AStackedLoding.Length;
            System.Array.Resize (ref AStackedLoding, len + 1);
            AStackedLoding[len] = Atlas;
            Atlas.readFromBytes (Ba, _pChar);
        }

        public static void clearLoadingStack () {
            AStackedLoding = null;
        }
        public int source_width {
            get { return Image.width; }
        }
        public int source_height {
            get { return Image.height; }
        }
        public int input_width {
            get { return Image.input_width; }
        }
        public int input_height {
            get { return Image.input_height; }
        }
        public Texture2D getImage () {
            return Image.Image;
        }
        public float getDrawScale () {
            return Image.scale;
        }

        public void dispose () {
            if (Image != null) Image.Unload ();
        }
        public static uint calcReso (uint v) {
            // 2のべき乗数に繰り上げ
            uint n = 1;
            while (v > n) {
                n = n << 1;
            }
            return n;
        }
        public static uint Pow (uint v, int n = 2) {
            uint v0 = v;
            v = 1;
            while (--n >= 0) {
                v *= v0;
            }
            return v;
        }
    }
}