﻿using System.Collections;
using System.Collections.Generic;
using PixelLiner;
using UnityEngine;

public class MyCharacter : MonoBehaviour {
	public float walkSpeed = 3.43f;
	public float ySpeedMax = -5.0f;
	public float ySpeedStart = 6.31f;
	public float ySpeedKeyReleased = 3.515f;

	PxlCharaAnimator TargetAnim;
	Rigidbody2D TargetRigidBody;
	int move_aim = 0; // 正 なら右、 負数 なら左へ
	bool is_on_ground = false;
	int jump_pushing = 0; // ジャンプ中(上方向入力を受け付ける)
	private int mvL = 0;
	private int mvR = 0;
	private int mvT = 0;

	private float positionx_min;
	private float positionx_max;

	void Start () {
		TargetAnim = GetComponent<PxlCharaAnimator> ();
		TargetRigidBody = GetComponent<Rigidbody2D> ();
		GameObject Ground = GameObject.Find ("Ground");
		positionx_max = (Ground.transform.localScale.x - 0.2f) / 2;
		positionx_min = -positionx_max;
	}

	// Update is called once per frame
	void Update () {
		// listen key state ====================================
		updateKey ();
		Vector2 velocity = TargetRigidBody.velocity;
		if (isRightOn () || isLeftOn ()) {
			if (isRightOn () && move_aim != -2) {
				move_aim = 1;
			}
			if (isLeftOn () && move_aim != 2) {
				move_aim = -1;
			}
		} else move_aim = 0;
		if (isRightPushDown ()) move_aim = 2;
		if (isLeftPushDown ()) move_aim = -2;
		if (isLeftUp () && move_aim == -2) move_aim = -1;
		if (isRightUp () && move_aim == 2) move_aim = 1;
		if (is_on_ground && isTopPushDown ()) {
			// ジャンプ開始
			jump_pushing = 7;
		}
		// move ====================================
		float xspeed = velocity.x;
		if (jump_pushing > 0) {
			is_on_ground = false;
			if (!isTopOn ()) jump_pushing = -1;
			else jump_pushing--;
		}
		if (velocity.y < -0.06f) {
			is_on_ground = false;
		}
		if (move_aim == 0) {
			// stopping
			xspeed = 0;
		} else {
			// walking
			xspeed = (move_aim > 0 ? 1 : -1) * walkSpeed;
		}
		velocity.x = xspeed;
		if (jump_pushing > 0) {
			velocity.y = ySpeedStart;
		} else {
			if (isTopUp () && velocity.y > ySpeedKeyReleased) {
				velocity.y = ySpeedKeyReleased;
			}
		}
		velocity.y = Mathf.Max (velocity.y, ySpeedMax);
		TargetRigidBody.velocity = velocity;
		if (velocity.x != 0) {
			Vector3 Pos = transform.position;
			Pos.x = Mathf.Clamp (Pos.x, positionx_min, positionx_max);
			transform.position = Pos;
		}
		// pose_manipulate ====================================
		string dep_pose;
		if (!is_on_ground) {
			dep_pose = velocity.y > 0 ? "jump" : "fall";
		} else {
			dep_pose = Mathf.Abs (xspeed) >= 0.1f ? "walk" : "stand";
		}
		TargetAnim.setPose (dep_pose);
		if (move_aim != 0) TargetAnim.setAim (move_aim > 0 ? PxlCharaAnimator.AIM_R : PxlCharaAnimator.AIM_L, 0);

	}

	void OnCollisionEnter2D (Collision2D col) {
		if (col.gameObject.tag == "Ground") {
			if (!is_on_ground) is_on_ground = true;
		}
	}
	void OnCollisionStay2D (Collision2D col) {
		if (col.gameObject.tag == "Ground") {
			if (!is_on_ground) is_on_ground = true;
		}
	}
	/* ___ input ___ */
	public void updateKey () {
		if (mvL != 0) mvL++;
		if (mvR != 0) mvR++;
		if (mvT != 0) mvT++;
		if (Input.GetKeyDown (KeyCode.A)) mvL = 1;
		else if (Input.GetKeyUp (KeyCode.A)) mvL = -1;

		if (Input.GetKeyDown (KeyCode.D)) mvR = 1;
		else if (Input.GetKeyUp (KeyCode.D)) mvR = -1;

		if (Input.GetKeyDown (KeyCode.W)) mvT = 1;
		else if (Input.GetKeyUp (KeyCode.W)) mvT = -1;

	}
	public bool isRightOn () {
		return mvR > 0;
	}
	public bool isLeftOn () {
		return mvL > 0;
	}
	public bool isRightPushDown () {
		return mvR == 1;
	}
	public bool isLeftPushDown () {
		return mvL == 1;
	}
	public bool isTopPushDown () {
		return mvT == 1;
	}
	public bool isTopOn () {
		return mvT > 0;
	}
	public bool isRightUp () {
		return mvR == -1;
	}
	public bool isLeftUp () {
		return mvL == -1;
	}
	public bool isTopUp () {
		return mvT == -1;
	}
}